/*
 *    SimpleGraphicalClient.java
 *    Copyright (C) 2001 New Zealand Digital Library Project
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.nzdl.gsdl.SimpleGraphicalClient;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.event.*; 
import java.io.*;


// local libraries
import org.nzdl.gsdl.service.NzdlCollectionInfo; 
import org.nzdl.gsdl.service.NzdlQuery;
import org.nzdl.gsdl.service.NzdlRequest;
import org.nzdl.gsdl.service.NzdlResponse;
import org.nzdl.gsdl.service.NzdlResultSet;
import org.nzdl.gsdl.service.NzdlService;
import org.nzdl.gsdl.service.NzdlServiceClient;


/**
 * A Class representing the Panel in which the Queryign action happens. 
 *
 * Does most of the actual `work' in the package.
 * 
 * @author Dave Nichols (daven@cs.waikato.ac.nz)
 * @author stuart yeates (say1@cs.waikato.ac.nz)
 * @version $Revision: 2145 $
 * @see org.nzdl.gsdl.service.SimpleGraphicalClient.SimpleGraphicalClient
 * @see org.nzdl.gsdl.service.SimpleGraphicalClient.CSModel
 * @see javax.swing.JPanel
 */

public class SearchPanel extends JPanel implements ActionListener, Constants
{

  CSModel csModel;
  JLabel collectionLabel;
  JTextField searchTextField;
  JButton searchButton;
  JComboBox collectionList;  // can we search multiple collections at once?
  JPanel queryFormulationPanel, resultsPanel, dataPanel; 
  JPanel collectionListPanel, searchTextFieldPanel, searchButtonPanel;
  /** where the ranked results are displayed */ 
  JList   resultsList;   
  JScrollPane scrollResultsPane;
  /** where the doc contents are displayed */
  JTextArea dataTextArea;  
  JScrollPane scrollDataPane;

  /**
   * Construct and initialise a new SearchPanel
   */

  public SearchPanel( CSModel newCsModel, JFrame parent) 
  {
    super();
    csModel = newCsModel;
    setLayout( new BoxLayout(this, BoxLayout.Y_AXIS));
    setBorder(BorderFactory.createTitledBorder("Search"));
	
	
    searchTextField = new JTextField("Enter search terms here", 35) { 
	public boolean isRequestFocusEnabled(){ 
	  return true; 
	}
      };


    searchTextField.setText("Enter search terms here");
    searchTextField.setFont(searchTextFieldFont);
    searchTextField.setColumns(35);
    searchTextField.setBorder(BorderFactory.createEmptyBorder(0,2,0,2));
    searchButton = new JButton("Search");
	
    searchButton.addActionListener(this);
    //searchButton.setBorder(BorderFactory.createEmptyBorder(2,2,2,2));
    searchButton.setPreferredSize(new Dimension(80, 20));
	
    collectionListPanel = new JPanel();
    collectionListPanel.setBorder(BorderFactory.createEmptyBorder(2,2,2,2));
    collectionListPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    collectionLabel = new JLabel("Collection: ");
    /*collectionList = new JComboBox(csModel.getCollectionList()){
      public boolean isFocusTraversable() {
      return false;
      }
      }; */
    //System.err.println("size is " + csModel.getCollectionListSize());
    //Collection coll = csModel.getCollectionList();
    //System.err.println("coll size is " + coll.size());
    //Vector vec = new Vector(coll);
    //System.err.println("vector size is " + vec.size());
    collectionList = new JComboBox(csModel.getCollectionList()){
	public boolean isFocusTraversable() {
	  return false;
	}
      };





    //collectionList.setSelectedIndex(0); // only if there is content
    collectionListPanel.add(collectionLabel);
    collectionListPanel.add(collectionList);

	
    searchTextFieldPanel = new JPanel();
    searchTextFieldPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    searchTextFieldPanel.add(searchTextField);
	
    searchButtonPanel = new JPanel();
    searchButtonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
    searchButtonPanel.add(searchButton);
	
    queryFormulationPanel = new JPanel();
    queryFormulationPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
    queryFormulationPanel.setLayout(new BoxLayout(queryFormulationPanel, BoxLayout.Y_AXIS));
    queryFormulationPanel.add(Box.createHorizontalStrut(100));
    queryFormulationPanel.add(collectionListPanel);
    queryFormulationPanel.add(searchTextFieldPanel);
    queryFormulationPanel.add(searchButtonPanel);
	
	
    resultsPanel = new JPanel();
    resultsPanel.setLayout(new BorderLayout());
    resultsPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
    ResultModel resultListModel = csModel.getResultsModel();
    resultsList = new JList(resultListModel); 
    resultsList.setFont(resultsFont);
    resultsList.setBorder(BorderFactory.createEmptyBorder(3,3,3,3));
    resultsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    scrollResultsPane = new JScrollPane(resultsList);
    resultsPanel.add(scrollResultsPane, BorderLayout.CENTER);
    csModel.addResult(new Result("Search results will appear here", "", ""));
	
    dataPanel = new JPanel();
    dataPanel.setLayout(new BorderLayout());
    dataPanel.setBorder(BorderFactory.createEmptyBorder(3,3,3,3));
    dataTextArea = new JTextArea("Document details will appear here", 10, 0);
    dataTextArea.setEditable(false);
    dataTextArea.setLineWrap(true);
    dataTextArea.setFont(docFont);
    dataTextArea.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
    scrollDataPane = new JScrollPane(dataTextArea);
    dataPanel.add(scrollDataPane, BorderLayout.CENTER);
    
    parent.getRootPane().setDefaultButton(searchButton);
	
    //resultsList.getSelectionModel().addListSelectionListener(new ResultListSelectionHandler() );
    resultsList.addListSelectionListener(new ResultListSelectionHandler() );
	
    add(Box.createHorizontalStrut(400));
	
    add(queryFormulationPanel);
    add(resultsPanel);
    add(dataPanel);
    searchTextField.requestFocus();
  }

  public void setCollectionIndex(int index) {

    collectionList.setSelectedIndex(index);

  }


  /** respond to the user pressing the Search button */

  public void actionPerformed(ActionEvent e) {
    Object source = e.getSource();
    if (source == searchButton ) { //assume only searchButton generates events
      System.out.println("Search button pressed in SearchPanel");
      //send query to collection
      NzdlQuery nzdlQuery = new NzdlQuery( searchTextField.getText() );
      // return the first numResults that match
      //query.setEndResults( 15 );
      // "-1" means consider all the documents that match
      nzdlQuery.setMaxDocs( -1 );
      String collectionName = collectionList.getSelectedItem().toString();
      NzdlRequest request = new NzdlRequest( nzdlQuery );
      NzdlResponse response = new NzdlResponse( );
      csModel.getNzdlService().service( collectionName, request, response ); //nzdl object ????
      NzdlResultSet results = response.getResultSet();
      java.util.List docIDs = results.getDocumentIDs();
      // paired collection of docIDs and Titles - but as Sets!
      Map titleMetaData = csModel.getNzdlService().getMetaData( collectionName, docIDs, "Title" );
      // created a paired collection of docIDs and Titles - as simple Strings!
      HashMap titleMap = new HashMap();
      for (ListIterator i = docIDs.listIterator(); i.hasNext(); ) {
	String docID = (String) i.next();
	Set titleSet = (Set) titleMetaData.get( docID );
	ArrayList titleList = new ArrayList(titleSet);
	// String documentContents = csModel.getNzdlService().getDocument(collectionName, docID);
	//titleMap.put(docID, titleSet.toString()); // 
	titleMap.put(docID, titleList.get(0)); // use the first title as *the* title
      } // end for
      // update resultsListModel
      // empty previous results List
      csModel.clearResults();
      System.err.println("Size of docIDs = " + docIDs.size());
      Iterator docIDsIterator = docIDs.iterator();
      while (docIDsIterator.hasNext()) {
	String resultDocID =  (String) docIDsIterator.next();
	Result result = new Result( (String)titleMap.get(resultDocID), resultDocID, collectionName );
	System.err.println("Adding result");
	csModel.addResult(result);
      } //end while
      // default to showing the top ranked document
      if (docIDs.size() > 0)
	resultsList.setSelectedIndex(0); 
    } // end if
    else {
      System.out.println("unknown Action event in SearchPanel");
    }
  } //end actionPerformed



  /**
   * handles selection events in the results list so that when 
   * the user selects a new title (say) the document contents 
   * is automatically displayed in the main panel 
   */

  class ResultListSelectionHandler implements ListSelectionListener {
    public void valueChanged(ListSelectionEvent e) {
      //get the result object from the SelectedIndex
      Result result = (Result) resultsList.getSelectedValue();
      // should check here to see if anything was actually selected !
      //get the document content from the result object
      // use a String for the moment to hold the contents
      String documentContents = csModel.getNzdlService().getDocument(result.getCollectionName(),  result.getDocID());
      dataTextArea.setText(documentContents);
      dataTextArea.setCaretPosition(0);
    } //end valueChanged
  }//end ResultListSelectionHandler


} //end SearchPanel
